;;; context.lisp ---
;;
;; Copyright (C) 2010 Jan Moringen
;;
;; Author: Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
;;
;; This Program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This Program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses>.

(in-package :gir.emitter)


;;; Context Protocol
;;

(defgeneric push-package (context name)
  (:documentation
   "Push package designated by NAME on the package stack of
CONTEXT."))

(defgeneric pop-package (context)
  (:documentation
   "Pop the top package off the package stack of CONTEXT."))

(defgeneric current-package (context)
  (:documentation
   "Return the current package of CONTEXT."))

(defvar *context*)

(declaim (special *context*))


;;; Class context
;;

(defclass context ()
  ((target        :initarg  :target
		  :type     symbol
		  :initform nil
		  :accessor context-target
		  :documentation
		  "Stores the current emitter target.")
   (stack         :initarg  :stack
		  :type     list
		  :initform nil
		  :accessor context-stack
		  :documentation
		  "Stack representing the current element nesting.")
   (package-stack :initarg  :package-stack
		  :type     list
		  :initform nil
		  :accessor context-package-stack
		  :documentation
		  "Stack representing the current package nesting."))
  (:documentation
   "Instance of this class represent the context in which code for a
introspection elements is emitted."))

(defmethod push-package ((context context) (name string))
  (push name (context-package-stack context))
  (current-package context))

(defmethod push-package ((context context) (name symbol))
  (push-package context (string name)))

(defmethod pop-package ((context context))
  (pop (context-package-stack context)))

(defmethod current-package ((context context))
  (bind (((:slots package-stack) context)
	 (designator (when package-stack
		       (intern
			(string-upcase
			 (reduce (lambda (lhs rhs)
				   (concatenate 'string lhs "." rhs))
				 package-stack))
			:keyword))))
	(when designator
	  (or (find-package designator)
	      (make-package designator)))))

(defvar *context* (make-instance 'context)
  "DOC")
