;;; generic.lisp ---
;;
;; Copyright (C) 2010 Jan Moringen
;;
;; Author: Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
;;
;; This Program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This Program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses>.

(in-package :gir.emitter)


;;; State Management
;;

(defmethod emit :around ((element t) (target1 t))
  (with-emitter-symbols
    (let ((old-target target))
      (setf target target1)
      (push element stack)
      (unwind-protect
	   (call-next-method)
	(pop stack)
	(setf target old-target)))))

(defmethod emit :around ((node namespace) (target t))
  "DOC"
  (with-emitter-symbols
    (bind (((:accessors (name node-name)) node)
	   (package-name (string-downcase name))) ;;(gobject::g-name->name name)
      (push-package package-name)
      (unwind-protect
	   (call-next-method)
	(pop-package)))))

(defmethod emit :before ((element t) (target t))
  (with-emitter-symbols
    (if element
	(let ((reversed (reverse stack)))
	  (format t "~{~{(~A ~A)~}~^ > ~}~%"
		  (mapcar #'list
			  (mapcar (compose #'class-name #'class-of) reversed)
			  (mapcar #'node-name reversed))))
	(warn "Element is nil~%"))))


;;;
;;

(defmethod emit ((node t) (target t))
  (warn "Skipping unknown element ~A~%" node))

(defmethod emit ((node composite) (target t))
  (traverse-children node))


;;; Restarts
;;

(defmacro make-restartable-emit (class &optional (name (string class)))
  ""
  (let ((retry-report-format (format nil "~~@<Retry emitting for ~A ~
node ~~A.~~@:>" name))
	(skip-report-format  (format nil "~~@<Do not emit anything for ~A ~
node ~~A and continue with next node~~@:>." name)))
    `(defmethod emit :around ((node ,class) (target t))
       ,(format nil "Run emit for node type ~A" class)
       (let (result)
	 (tagbody
	  start
	    (restart-case
		(progn
		  (call-next-method)
		  (go end))
	      (retry ()
		:report (lambda (stream)
			  (format stream ,retry-report-format node))
		(go start))
	      (skip ()
		:report (lambda (stream)
			  (format stream ,skip-report-format node))
		(go end)))
	  end)
	 result))))

(make-restartable-emit function1  "function")
(make-restartable-emit class1     "class")
(make-restartable-emit interface)
(make-restartable-emit record)
(make-restartable-emit enumeration)


;;; Utility Functions
;;

(defun traverse-children (element &key (test (constantly t)))
  "Call emit with TARGET on all children of ELEMENT."
  (with-emitter-symbols
    (iter (for child in (composite-children element))
	  (when (funcall test child)
	    (collect (recur child))))))
